'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.execute = exports.buildMatchIndex = exports.findMatchOffsets = exports.findSpeechOffsets = exports.matchesToIntervals = undefined;

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _immutable = require('immutable');

var _intervalTree1d = require('interval-tree-1d');

var _intervalTree1d2 = _interopRequireDefault(_intervalTree1d);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var matchesToIntervals = exports.matchesToIntervals = function matchesToIntervals(matches) {
  return matches.map(function (match) {
    return _immutable.List.of(match.index, match.index + match[0].length);
  });
};

var findSpeechOffsets = exports.findSpeechOffsets = function findSpeechOffsets(speech) {
  return speech.reduce(function (_ref, segment) {
    var _ref2 = (0, _slicedToArray3.default)(_ref, 2);

    var start = _ref2[0];
    var offsets = _ref2[1];

    var end = start + segment.get('text').length;
    offsets.push([start, end]);
    return [end + 1, offsets];
  }, [0, []])[1];
};

var findMatchOffsets = exports.findMatchOffsets = function findMatchOffsets(speech, matches) {
  var speechOffsets = findSpeechOffsets(speech),
      tree = (0, _intervalTree1d2.default)(speechOffsets);
  return matchesToIntervals(matches).map(function (_ref3) {
    var _ref4 = (0, _slicedToArray3.default)(_ref3, 2);

    var start = _ref4[0];
    var end = _ref4[1];

    var matchOffsets = (0, _immutable.Map)();
    tree.queryInterval(start, end, function (offset) {
      var i = speechOffsets.indexOf(offset);
      matchOffsets = matchOffsets.set(i, _immutable.List.of(Math.max(0, start - offset[0]), Math.min(offset[1] - offset[0], end - offset[0])));
    });
    return matchOffsets;
  });
};

var buildMatchIndex = exports.buildMatchIndex = function buildMatchIndex(turns, re) {
  return turns.reduce(function (index, turn, turn_idx) {
    var text = turn.get('sentences').join(' ');
    var match = [],
        matches = (0, _immutable.List)();
    if (re !== null) {
      while ((match = re.exec(text)) !== null) {
        matches = matches.push(match);
      }
    }
    if (matches.size > 0) {
      index = index.set(turn_idx, findMatchOffsets(turn.get('speech'), matches));
    }
    return index;
  }, (0, _immutable.Map)());
};

var execute = exports.execute = function execute(turns) {
  var re = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

  turns = (0, _immutable.fromJS)(turns);
  var index = buildMatchIndex(turns, re);
  var times = (0, _immutable.List)();
  index.keySeq().sort().forEach(function (turn_idx) {
    var matches = index.get(turn_idx);
    times = times.concat(matches.map(function (match) {
      var speech_idx = match.keySeq().sort().first();
      return turns.get(turn_idx).get('speech').get(speech_idx).get('start');
    }));
  });
  return (0, _immutable.Map)({ matches: index, times: times });
};